from django.core.management.base import BaseCommand
from eveuniverse.models import EveSolarSystem, EveType


def get_input(text):
    """wrapped input to enable unit testing / patching"""
    return input(text)


class Command(BaseCommand):
    help = "Setup all needed data for buyback program to operate"

    def _update_models(self):

        self.stdout.write(
            "Adding %s objects to celery task queue. This may take a minute..."
            % (EveSolarSystem.__name__)
        )

        EveSolarSystem.objects.update_or_create_all_esi(
            include_children=False, wait_for_children=False
        )

        self.stdout.write(
            "Adding %s objects to celery task queue. This may take a few minutes..."
            % (EveType.__name__)
        )

        EveType.objects.update_or_create_all_esi(
            include_children=False,
            wait_for_children=False,
            enabled_sections=[
                EveType.Section.TYPE_MATERIALS,
                EveType.Section.MARKET_GROUPS,
            ],
        )

    def handle(self, *args, **options):
        self.stdout.write(
            "This command will load all the required data needed for the program to operate."
            "This process will take a very long time as we are loading every single items and solar system from ESI."
        )
        user_input = get_input("Are you sure you want to proceed? (y/N)?")
        if user_input.lower() == "y":
            self._update_models()
            self.stdout.write(
                "All objects added to celery queue. You can monitor the progress of these tasks from your AUTH dashboard. You can expect to see ~80k tasks added to the que. Once tasks are completed you can preload price data with buybackprogram_load_prices"
            )
        else:
            self.stdout.write("Aborted")



# Generated by Django 3.2.10 on 2022-01-25 09:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("buybackprogram", "0002_contractnotification"),
    ]

    operations = [
        migrations.AddField(
            model_name="program",
            name="name",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A name or a description for this program",
                max_length=64,
                verbose_name="Name/description",
            ),
        ),
        migrations.AddField(
            model_name="tracking",
            name="contract",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="buybackprogram.contract",
            ),
        ),
        migrations.AddField(
            model_name="tracking",
            name="created_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



import requests

from django.core.management.base import BaseCommand
from django.db import IntegrityError
from django.utils import timezone
from eveuniverse.models import EveType

from allianceauth.services.hooks import get_extension_logger

from buybackprogram.models import ItemPrices

logger = get_extension_logger(__name__)


class Command(BaseCommand):
    help = (
        "Preloads price data required for the buyback program from fuzzwork market API"
    )

    def handle(self, *args, **options):
        i = 0
        item_count = 0
        type_ids = []
        market_data = []

        # Get all type ids
        typeids = EveType.objects.values_list("id", flat=True).filter(published=True)

        print(
            "Price setup starting for %s items from Fuzzworks API, this may take up to 30 seconds..."
            % len(typeids)
        )

        # Build suitable bulks to fetch prices from API
        for item in typeids:
            type_ids.append(item)

            i += 1

            if i == 1000:

                response_fuzzwork = requests.get(
                    "https://market.fuzzwork.co.uk/aggregates/",
                    params={
                        "types": ",".join([str(x) for x in type_ids]),
                        "station": 60003760,
                    },
                )

                items_fuzzwork = response_fuzzwork.json()
                market_data.append(items_fuzzwork)

                i = 0
                type_ids.clear()

        # Get leftover data from the bulk
        response_fuzzwork = requests.get(
            "https://market.fuzzwork.co.uk/aggregates/",
            params={
                "types": ",".join([str(x) for x in type_ids]),
                "station": 60003760,
            },
        )

        items_fuzzwork = response_fuzzwork.json()
        market_data.append(items_fuzzwork)

        objs = []

        for objects in market_data:
            for key, value in objects.items():
                item_count += 1

                item = ItemPrices(
                    eve_type_id=key,
                    buy=int(float(value["buy"]["max"])),
                    sell=int(float(value["sell"]["min"])),
                    updated=timezone.now(),
                )

                objs.append(item)
        try:
            ItemPrices.objects.bulk_create(objs)
            return "Succesfully setup %s prices." % item_count
        except IntegrityError:
            print(
                "Error: Prices already loaded into database, did you mean to run task.update_all_prices instead?"
            )

            delete_arg = input("Would you like to delete current prices? (y/n): ")

            if delete_arg == "y":
                ItemPrices.objects.all().delete()
                return "All price data removed from database. Run the command again to populate the price data."
            else:
                return "No changes done to price table."


# from buybackprogram.tasks import update_all_prices



from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import Q
from django.shortcuts import render
from eveuniverse.models import EveEntity

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.services.hooks import get_extension_logger

from ..models import (
    Contract,
    ContractItem,
    ContractNotification,
    Tracking,
    TrackingItem,
)

logger = get_extension_logger(__name__)


@login_required
@permission_required("buybackprogram.basic_access")
def my_stats(request):

    # List for valid contracts to be displayed
    valid_contracts = []

    # Tracker values
    values = {
        "outstanding": 0,
        "finished": 0,
        "outstanding_count": 0,
        "finished_count": 0,
    }

    # Request user owned characters
    characters = CharacterOwnership.objects.filter(user=request.user).values_list(
        "character__character_id", flat=True
    )

    # Get all tracking objects that have a linked contract to them for the user
    tracking_numbers = (
        Tracking.objects.filter(contract__isnull=False)
        .filter(contract__issuer_id__in=characters)
        .prefetch_related("contract")
    )

    # Loop tracking objects to see if we have any contracts
    for tracking in tracking_numbers:

        # Get notes for this contract
        tracking.contract.notes = ContractNotification.objects.filter(
            contract=tracking.contract
        )

        # Walk the tracker values for contracts
        if tracking.contract.status == "outstanding":
            values["outstanding"] += tracking.contract.price
            values["outstanding_count"] += 1
        if tracking.contract.status == "finished":
            values["finished"] += tracking.contract.price
            values["finished_count"] += 1

        # Get the name for the issuer
        tracking.contract.issuer_name = EveEntity.objects.resolve_name(
            tracking.contract.issuer_id
        )

        # Get the name for the assignee
        tracking.contract.assignee_name = EveEntity.objects.resolve_name(
            tracking.contract.assignee_id
        )

        # Add contract to the valid contract list
        valid_contracts.append(tracking)

    context = {
        "contracts": valid_contracts,
        "values": values,
        "mine": True,
    }

    return render(request, "buybackprogram/stats.html", context)


@login_required
@permission_required("buybackprogram.manage_programs")
def program_stats(request):

    # List for valid contracts to be displayed
    valid_contracts = []

    # Tracker values
    values = {
        "outstanding": 0,
        "finished": 0,
        "outstanding_count": 0,
        "finished_count": 0,
    }

    # Request user owned characters
    characters = CharacterOwnership.objects.filter(user=request.user).values_list(
        "character__character_id", flat=True
    )

    # Request user owned corporations
    corporations = CharacterOwnership.objects.filter(user=request.user).values_list(
        "character__corporation_id", flat=True
    )

    # Get all tracking objects that have a linked contract to them for the user
    tracking_numbers = (
        Tracking.objects.filter(contract__isnull=False)
        .filter(
            Q(contract__assignee_id__in=characters)
            | Q(contract__assignee_id__in=corporations)
        )
        .prefetch_related("contract")
    )

    # Loop tracking objects to see if we have any contracts
    for tracking in tracking_numbers:

        # Get notes for this contract
        tracking.contract.notes = ContractNotification.objects.filter(
            contract=tracking.contract
        )

        # Walk the tracker values for contracts
        if tracking.contract.status == "outstanding":
            values["outstanding"] += tracking.contract.price
            values["outstanding_count"] += 1
        if tracking.contract.status == "finished":
            values["finished"] += tracking.contract.price
            values["finished_count"] += 1

        # Get the name for the issuer
        tracking.contract.issuer_name = EveEntity.objects.resolve_name(
            tracking.contract.issuer_id
        )

        # Get the name for the assignee
        tracking.contract.assignee_name = EveEntity.objects.resolve_name(
            tracking.contract.assignee_id
        )

        # Add contract to the valid contract list
        valid_contracts.append(tracking)

    context = {
        "contracts": valid_contracts,
        "values": values,
        "mine": True,
    }

    return render(request, "buybackprogram/stats.html", context)


@login_required
@permission_required("buybackprogram.see_all_statics")
def program_stats_all(request):

    # List for valid contracts to be displayed
    valid_contracts = []

    # Tracker values
    values = {
        "outstanding": 0,
        "finished": 0,
        "outstanding_count": 0,
        "finished_count": 0,
    }

    # Get all tracking objects that have a linked contract to them for the user
    tracking_numbers = Tracking.objects.filter(contract__isnull=False).prefetch_related(
        "contract"
    )

    # Loop tracking objects to see if we have any contracts
    for tracking in tracking_numbers:

        # Get notes for this contract
        tracking.contract.notes = ContractNotification.objects.filter(
            contract=tracking.contract
        )

        # Walk the tracker values for contracts
        if tracking.contract.status == "outstanding":
            values["outstanding"] += tracking.contract.price
            values["outstanding_count"] += 1
        if tracking.contract.status == "finished":
            values["finished"] += tracking.contract.price
            values["finished_count"] += 1

        # Get the name for the issuer
        tracking.contract.issuer_name = EveEntity.objects.resolve_name(
            tracking.contract.issuer_id
        )

        # Get the name for the assignee
        tracking.contract.assignee_name = EveEntity.objects.resolve_name(
            tracking.contract.assignee_id
        )

        valid_contracts.append(tracking)

    context = {
        "contracts": valid_contracts,
        "values": values,
        "mine": True,
    }

    return render(request, "buybackprogram/stats.html", context)


@login_required
@permission_required("buybackprogram.basic_access")
def contract_details(request, contract_title):

    contract = Contract.objects.get(title__contains=contract_title)

    # Get notes for this contract
    notes = ContractNotification.objects.filter(contract=contract)

    # Get items for this contract
    contract_items = ContractItem.objects.filter(contract=contract)

    # Get tracking object for this contract
    tracking = Tracking.objects.get(
        tracking_number=contract_title,
    )

    # Get tracked items
    tracking_items = TrackingItem.objects.filter(tracking=tracking)

    # Get the name for the issuer
    contract.issuer_name = EveEntity.objects.resolve_name(contract.issuer_id)

    # Get the name for the assignee
    contract.assignee_name = EveEntity.objects.resolve_name(contract.assignee_id)

    context = {
        "notes": notes,
        "contract": contract,
        "contract_items": contract_items,
        "tracking": tracking,
        "tracking_items": tracking_items,
    }

    return render(request, "buybackprogram/contract_details.html", context)
